"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .subscription_output import SubscriptionOutput, SubscriptionOutputTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceSubscriptionTypedDict(TypedDict):
    items: List[SubscriptionOutputTypedDict]
    pagination: PaginationTypedDict


class ListResourceSubscription(BaseModel):
    items: List[SubscriptionOutput]

    pagination: Pagination
