"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SubscriptionTierType(str, Enum):
    FREE = "free"
    INDIVIDUAL = "individual"
    BUSINESS = "business"
