"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import TypedDict


class SubscriptionsImportedTypedDict(TypedDict):
    r"""Result of a subscription import operation."""

    count: int


class SubscriptionsImported(BaseModel):
    r"""Result of a subscription import operation."""

    count: int
