"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitgrant import BenefitGrant, BenefitGrantTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceBenefitGrantTypedDict(TypedDict):
    items: List[BenefitGrantTypedDict]
    pagination: PaginationTypedDict


class ListResourceBenefitGrant(BaseModel):
    items: List[BenefitGrant]

    pagination: Pagination
