"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class UserAdvertisementSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    VIEWS = "views"
    MINUS_VIEWS = "-views"
    CLICKS = "clicks"
    MINUS_CLICKS = "-clicks"
