"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .downloadableread import DownloadableRead, DownloadableReadTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceDownloadableReadTypedDict(TypedDict):
    items: List[DownloadableReadTypedDict]
    pagination: PaginationTypedDict


class ListResourceDownloadableRead(BaseModel):
    items: List[DownloadableRead]

    pagination: Pagination
